import json
import sys

sys.path.append('../')

def get_message(client):
    """Функция обработки сообщения"""

    encoded_response = client.recv(1024)
    if type(encoded_response) == bytes:
        json_response = encoded_response.decode('utf-8')
        response = json.loads(json_response)
        if type(response) == dict:
            return response
        else:
            raise ValueError
    else:
        raise ValueError

def send_message(sock, message):
    """Функция отправки сообщения в формате json"""

    if not isinstance(message, dict):
        raise ValueError
    js_message = json.dumps(message)
    encoded_message = js_message.encode('utf-8')
    sock.send(encoded_message)


import os
import sys

from sqlalchemy import create_engine, Table, Column, Integer, String, MetaData, ForeignKey, DateTime, Text
from sqlalchemy.orm import mapper, sessionmaker
import datetime
from sqlalchemy.sql import default_comparator

sys.path.append('../../../')


class ClientStorage:
    """Класс для работы с базой данной клиента"""
    class KnownUsers:
        """Класс таблица - известные пользователи"""
        def __init__(self, user):
            self.id = None
            self.username = user

    class MessageHistory:
        """Класс таблица - исптория сообщений"""
        def __init__(self, from_user, to_user, message):
            self.id = None
            self.from_user = from_user
            self.to_user = to_user
            self.message = message
            self.date = datetime.datetime.now()

    class Contacts:
        """Класс таблица - контакты"""
        def __init__(self, contact):
            self.id = None
            self.name = contact

    def __init__(self, name):
        path = os.path.dirname(os.getcwd())
        filename = f'client_{name}.db3'
        self.engine = create_engine(
            f'sqlite:///{os.path.join(path, filename)}',
            echo=False,
            pool_recycle=7200,
            connect_args={
                'check_same_thread': False})

        self.metadata = MetaData()

        users = Table('known_users', self.metadata,
                      Column('id', Integer, primary_key=True),
                      Column('username', String)
                      )

        history = Table('message_history', self.metadata,
                        Column('id', Integer, primary_key=True),
                        Column('from_user', String),
                        Column('to_user', String),
                        Column('message', Text),
                        Column('date', DateTime)
                        )

        contacts = Table('contacts', self.metadata,
                         Column('id', Integer, primary_key=True),
                         Column('name', String, unique=True)
                         )

        self.metadata.create_all(self.engine)

        mapper(self.KnownUsers, users)
        mapper(self.MessageHistory, history)
        mapper(self.Contacts, contacts)

        Session = sessionmaker(bind=self.engine)
        self.session = Session()

        self.session.query(self.Contacts).delete()
        self.session.commit()

    def add_contact(self, contact):
        """Добавление контакта"""
        if not self.session.query(
                self.Contacts).filter_by(
                name=contact).count():
            contact_ = self.Contacts(contact)
            self.session.add(contact_)
            self.session.commit()

    def contacts_clear(self):
        """Очищаем таблицу списка контактов"""
        self.session.query(self.Contacts).delete()

    def del_contact(self, contact):
        """Удаление контакта"""
        self.session.query(self.Contacts).filter_by(name=contact).delete()

    def add_users(self, users_list):
        """Добавление известных пользователей"""
        self.session.query(self.KnownUsers).delete()
        for user in users_list:
            user_row = self.KnownUsers(user)
            self.session.add(user_row)
        self.session.commit()

    def get_contacts(self):
        """Запрос списка контактов"""
        return [contact[0]
                for contact in self.session.query(self.Contacts.name).all()]

    def get_users(self):
        """Список известных пользователей"""
        return [user[0]
                for user in self.session.query(self.KnownUsers.username).all()]

    def check_user(self, user):
        """
        Проверка пользователя на присутствие
        в списке известных пользователей
        """
        if self.session.query(
                self.KnownUsers).filter_by(
                username=user).count():
            return True
        else:
            return False

    def save_message(self, from_user, to_user, message):
        """Сохраненение сообщений"""
        message_row = self.MessageHistory(from_user, to_user, message)
        self.session.add(message_row)
        self.session.commit()

    def check_contact(self, contact):
        """
        Проверка пользователя на присутствие
        в списке контактов
        """
        if self.session.query(self.Contacts).filter_by(name=contact).count():
            return True
        else:
            return False

    def get_history(self, from_who):
        """История переписки"""
        query = self.session.query(
            self.MessageHistory).filter_by(
            from_user=from_who)
        return [(history_row.from_user,
                 history_row.to_user,
                 history_row.message,
                 history_row.date) for history_row in query.all()]


# отладка
if __name__ == '__main__':
    test_db = ClientStorage('test1')
    for i in ['test3', 'test4', 'test5']:
        test_db.add_contact(i)
    test_db.add_contact('test4')
    test_db.add_users(['test1', 'test2', 'test3', 'test4', 'test5'])
    test_db.save_message(
        'test1',
        'test2',
        f'Тестовое сообщение ---- 1. Отправлено : {datetime.datetime.now()}!')
    test_db.save_message(
        'test2',
        'test1',
        f'Тестовое сообщение ---- 2. Отправлено : {datetime.datetime.now()}!')
    print(test_db.get_contacts())
    print(test_db.get_users())
    print(test_db.check_user('test1'))
    print(test_db.check_user('test10'))
    print(test_db.get_history('test2'))
    print(sorted(test_db.get_history('test2'), key=lambda item: item[3]))
    print(test_db.get_history('test3'))
    test_db.del_contact('test4')
    print(test_db.get_contacts())



# Прежде чем запускать тестирование , необходимо запустить
# файл server.py
# Я пытался запустить ,как отдельный процесс в самом скрипте,
# но регулярно сталкивался с ошибкой :
# ResourceWarning: Enable tracemalloc to get the object allocation traceback
# возможно Вы мне сможете подсказать как ее обойти.


import unittest
from socket import socket, AF_INET, SOCK_STREAM

from server.server.common import utils


class UtilsTest(unittest.TestCase):
    test_message = {
        'action': 'presence',
        'time': 1,
        'user': {
            'account_name': 'Guest'
        }
    }

    def setUp(self):
        #subprocess.Popen([sys.executable, '../server.py'], shell=True)
        self.client_sock = socket(AF_INET, SOCK_STREAM)
        self.client_sock.connect(('localhost', 8080))

    def tearDown(self):
        self.client_sock.close()

    def test_get_message(self):
        """Тест обработки сообщения"""
        utils.send_message(self.client_sock, self.test_message)
        self.assertEqual(utils.get_message(client=self.client_sock), {'response': 200})

    def test_send_message(self):
        """ Тест отправки сообщения """
        utils.send_message(self.client_sock, self.test_message)
        self.assertEqual(self.client_sock.recv(1024).decode('utf-8'), '{"response": 200}')




import dis


class ServerVerifier(type):
    """
    Метакласс контроля использования параметров
    'SOCK_STREAM' и 'AF_INET' в классах серверной части
    """
    def __init__(self, clsname, bases, clsdict):

        methods_lst = []
        attr_lst = []
        for func in clsdict:
            try:
                instruction = dis.get_instructions(clsdict[func])
            except TypeError:
                pass
            else:
                for i in instruction:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods_lst:
                            methods_lst.append(i.argval)
                    elif i.opname == 'LOAD_ATTR':
                        if i.argval not in attr_lst:
                            attr_lst.append(i.argval)
        if 'connect' in methods_lst:
            raise TypeError('Использование метода connect недопустимо в серверном классе')
        if not 'SOCK_STREAM' in attr_lst and 'AF_INET' in attr_lst:
            raise TypeError('Некорректная инициализация сокета.')
        super().__init__(clsname, bases, clsdict)




class ClientVerifier(type):
    """
    Метакласс валидации использования методов
    в классах клиентской части
    """
    def __init__(self, clsname, bases, clsdict):
        methods_lst = []
        for func in clsdict:
            try:
                instruction = dis.get_instructions(clsdict[func])
            except TypeError:
                pass
            else:
                for i in instruction:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods_lst:
                            methods_lst.append(i.argval)
        for command in ('accept', 'listen', 'socket'):
            if command in methods_lst:
                raise TypeError("Методы : 'accept', 'listen', 'socket' нельзя использваоть в классе Client")

        if 'get_message' in methods_lst or 'send_message' in methods_lst:
            pass
        else:
            raise TypeError('В классе Client должны быть определенный методы :"get_message" или "send_message" ')
        super().__init__(clsname, bases, clsdict)


import unittest
from server.server import server


class TestServerHandler(unittest.TestCase):
    correct_data = {'response': 200}
    incorrect_data = {
        'response': 400,
        'error': 'Bad Request'
    }

    def test_no_action(self):
        self.assertEqual(server.client_requests_handler(
            {'time': '1.1', 'user': {'account_name': 'Guest'}}), self.incorrect_data)

    def test_wrong_action(self):
        self.assertEqual(server.client_requests_handler(
            {'action': 'post' ,'time': '1.1',  'user': {'account_name': 'Ivan'}}), self.incorrect_data)

    def test_no_time(self):
        self.assertEqual(server.client_requests_handler(
            {'action': 'presence', 'user': {'account_name': 'Guest'}}), self.incorrect_data)

    def test_no_user(self):
        self.assertEqual(server.client_requests_handler(
            {'action': 'presence', 'time': '1.1'}), self.incorrect_data)

    def test_wrong_user(self):
        self.assertEqual(server.client_requests_handler(
            {'action': 'presence','time': '1.1',  'user': {'account_name': 'Ivan'}}), self.incorrect_data)

    def test_correct(self):
        self.assertEqual(server.client_requests_handler(
            {'action': 'presence','time': '1.1',  'user': {'account_name': 'Guest'}}), self.correct_data)




if __name__ == '__main__':
    unittest.main()


import logging.handlers
import os
import sys

sys.path.append('../')

# путь к папке с фалом
PATH = os.path.join(os.getcwd(), 'log_files/client.log')

# создаем объект форматирования
CLIENT_FORMATTER = logging.Formatter('%(asctime)s %(levelname)s %(filename)s %(message)s')

# параметры логов (путь, формат, уровень, ротация)
SYSTEM_HANDLER = logging.StreamHandler(sys.stderr)
SYSTEM_HANDLER.setFormatter(CLIENT_FORMATTER)
SYSTEM_HANDLER.setLevel(logging.ERROR)
FILE_TO_LOGGING = logging.handlers.TimedRotatingFileHandler(PATH, encoding='utf8', interval=1, when='D')
FILE_TO_LOGGING.setFormatter(CLIENT_FORMATTER)

# создание логов
LOG_OBJ = logging.getLogger('client')
LOG_OBJ.addHandler(SYSTEM_HANDLER)
LOG_OBJ.addHandler(FILE_TO_LOGGING)
LOG_OBJ.setLevel(logging.DEBUG)

if __name__ == '__main__':
    LOG_OBJ.critical('Crtical error')
    LOG_OBJ.error('Error')
    LOG_OBJ.debug('Debug information')
    LOG_OBJ.info('Info message')




import unittest
from client.client import client


class TestClient(unittest.TestCase):
    """Класс тестирования"""

    correct_result = {
        'action': 'presence',
        'time': 1,
        'user': {
            'account_name': 'Guest'
        }
    }

    test_create_presence = client.create_presence()
    test_create_presence['time'] = 1

    def test_type_result(self):
        self.assertIsInstance(client.create_presence(), dict)

    def test_correct(self):
        self.assertEqual(self.test_create_presence, self.correct_result)

    def test_response_200(self):
        self.assertEqual(client.server_response_handler({'response': 200}), '200 : OK')

    def test_response_400(self):
        self.assertEqual(client.server_response_handler(
            {'response': 400, 'error': 'Bad Request'}), '400 : Bad Request')



if __name__ == '__main__':
    unittest.main()



import logging
import os
from datetime import datetime
import inspect


if os.getcwd().split('/')[-1].split('.')[0] == 'client':
    LOGGER = logging.getLogger('client')
else:
    LOGGER = logging.getLogger('server')


def log(func):
    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        LOGGER.debug(f"Обращение к функции : {func.__name__} с аргументами {args} {kwargs}."
                     f"Функция {func.__name__} была вызвана : {datetime.now().strftime('%d-%m-%Y %H:%M')}"
                     f" из функции {inspect.stack()[1][3]}")
        return result
    return wrapper


class Log:
    def __call__(self, func):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            LOGGER.debug(f"Обращение к функции : {func.__name__} с аргументами {args} {kwargs}."
                         f"Функция {func.__name__} была вызвана : {datetime.now().strftime('%d-%m-%Y %H:%M')}"
                         f" из функции {inspect.stack()[1][3]}")
            return result
        return wrapper






from server.server.common.utils import get_message, send_message
from socket import socket, AF_INET, SOCK_STREAM
import time
import logging
import json
import threading
import hashlib
import hmac
import binascii
from PyQt5.QtCore import pyqtSignal, QObject


# логгер
client_logger = logging.getLogger('client')
# объект блокировки сокета
socket_lock = threading.Lock()


class ClientConnection(threading.Thread, QObject):
    """Класс - взаимодействие с сервером"""

    new_message = pyqtSignal(dict)
    message_205 = pyqtSignal()
    connection_lost = pyqtSignal()

    def __init__(self, port, ip_address, database, username, password, keys):
        threading.Thread.__init__(self)
        QObject.__init__(self)

        self.database = database
        self.username = username
        self.password = password
        self.client_sock = None
        self.keys = keys
        self.connection_init(port, ip_address)
        try:
            self.user_list_update()
            self.contacts_list_update()
        except OSError:
            client_logger.error(
                'Timeout соединения при обновлении списков пользователей.')
        except json.JSONDecodeError:
            client_logger.critical(f'Потеряно соединение с сервером.')
            raise Exception('Потеряно соединение с сервером!')
        # флаг продолжение работы сокета
        self.running = True

    def connection_init(self, port, ip):
        """Инициализация соединения"""
        self.client_sock = socket(AF_INET, SOCK_STREAM)
        self.client_sock.settimeout(10)

        connected = False
        for i in range(5):
            client_logger.info(f'Попытка подключения №{i + 1}')
            try:
                self.client_sock.connect((ip, port))
            except (OSError, ConnectionRefusedError):
                client_logger.debug(
                    f"ошибка при попытке соединения - client_sock.connect")
                pass
            else:
                connected = True
                client_logger.debug("Connection established")
                break
            time.sleep(1)

        if not connected:
            client_logger.critical(
                'Не удалось установить соединение с сервером')
            raise Exception('Не удалось установить соединение с сервером')

        client_logger.debug('Установлено соединение с сервером')

        # получаем хэш пароль
        password_bytes = self.password.encode('utf-8')
        salt = self.username.lower().encode('utf-8')
        password_hash = hashlib.pbkdf2_hmac(
            'sha512', password_bytes, salt, 10000)
        password_hash_str = binascii.hexlify(password_hash)

        client_logger.debug(f'Password hash ready: {password_hash_str}')

        # публичный ключ
        pubkey = self.keys.publickey().export_key().decode('ascii')
        client_logger.debug(f"Публичный ключ готов : {pubkey}")

        # авторизация

        with socket_lock:
            presence = {
                'action': 'presence',
                'time': time.time(),
                'user': {
                    'account_name': self.username,
                    'pubkey': pubkey
                }
            }
            client_logger.debug(f"Presence message = {presence}")

            try:
                send_message(self.client_sock, presence)
                answer = get_message(self.client_sock)
                client_logger.debug(f'Server response = {answer}.')

                if 'response' in answer:
                    if answer['response'] == 400:
                        client_logger.critical(
                            f"400 ответ сервера - ошибка : {answer['error']}")
                    elif answer['response'] == 511:
                        answer_data = answer['bin']
                        hash_ = hmac.new(
                            password_hash_str, answer_data.encode('utf-8'), 'MD5')
                        digest = hash_.digest()
                        my_answer = {'response': 511, 'bin': None}
                        my_answer['bin'] = binascii.b2a_base64(
                            digest).decode('ascii')
                        send_message(self.client_sock, my_answer)
                        self.server_response_handler(
                            get_message(self.client_sock))
            except (OSError, json.JSONDecodeError) as err:
                client_logger.debug(f'Connection error.', exc_info=err)
                raise ConnectionError

    def create_presence(self):
        """Формируем сообщение присутствия"""
        result = {
            'action': 'presence',
            'time': time.time(),
            'user': {
                'account_name': self.username
            }
        }
        client_logger.info(
            f'Сформировано "presence" сообщение для пользователя {self.username}')
        return result

    def server_response_handler(self, message):
        """Обработка сообщения-присутствия сервера"""
        client_logger.debug(f'Разбор сообщения от сервера: {message}')
        if 'response' in message:
            if message['response'] == 200:
                return '200 : OK'
            elif message['response'] == 400:
                return f'400 : {message["error"]}'
            elif message['response'] == 205:
                self.user_list_update()
                self.contacts_list_update()
                self.message_205.emit()
            else:
                client_logger.error(
                    f'Принят неизвестный код подтверждения {message["response"]}')

        elif 'action' in message and message['action'] == 'message' and 'from' in message and 'to' in message \
             and 'message_text' in message and message['to'] == self.username:
            client_logger.debug(
                f'Получено сообщение от пользователя {message["from"]}:{message["message_text"]}')
            self.new_message.emit(message)

    def contacts_list_update(self):
        '''Метод обновляющий с сервера список контактов.'''
        self.database.contacts_clear()
        client_logger.debug(
            f'Запрос контакт листа для пользователя {self.name}')
        request = {
            'action': 'get_contacts',
            'time': time.time(),
            'user': self.username
        }
        client_logger.debug(f'Сформирован запрос {request}')
        with socket_lock:
            send_message(self.client_sock, request)
            answer = get_message(self.client_sock)
        client_logger.debug(f'Получен ответ {answer}')
        if 'response' in answer and answer['response'] == 202:
            for contact in answer['data_list']:
                self.database.add_contact(contact)
        else:
            client_logger.error('Не удалось обновить список контактов.')

    def user_list_update(self):
        client_logger.debug(f'Запрос списка пользователей {self.username}')
        request = {
            'action': 'get_users',
            'time': time.time(),
            'account_name': self.username
        }
        with socket_lock:
            send_message(self.client_sock, request)
            answer = get_message(self.client_sock)
        if 'response' in answer and answer['response'] == 202:
            self.database.add_users(answer['data_list'])
        else:
            client_logger.error(
                'Не удалось обновить список известных пользователей.')

    def add_contact(self, contact):
        """Добавление контакта"""
        client_logger.debug(f'Создание контакта {contact}')
        request = {
            'action': 'add',
            'time': time.time(),
            'user': self.username,
            'account_name': contact
        }
        with socket_lock:
            send_message(self.client_sock, request)
            self.server_response_handler(get_message(self.client_sock))

    def remove_contact(self, contact):
        """Удаление контакта"""
        client_logger.debug(f'Удаление контакта {contact}')
        request = {
            'action': 'remove',
            'time': time.time(),
            'user': self.username,
            'account_name': contact
        }
        with socket_lock:
            send_message(self.client_sock, request)
            self.server_response_handler(get_message(self.client_sock))

    def transport_shutdown(self):
        """Уведомление о завершении работы клиента"""
        self.running = False
        message = {
            'action': 'exit',
            'time': time.time(),
            'account_name': self.username
        }
        with socket_lock:
            try:
                send_message(self.client_sock, message)
            except OSError:
                pass
        client_logger.debug('Транспорт завершает работу.')
        time.sleep(0.5)

    def key_request(self, user):
        '''Метод запрашивающий с сервера публичный ключ пользователя.'''
        client_logger.debug(f'Запрос публичного ключа для {user}')
        request = {
            'action': 'pubkey_need',
            'time': time.time(),
            'account_name': user
        }
        with socket_lock:
            send_message(self.client_sock, request)
            answer = get_message(self.client_sock)
        if 'response' in answer and answer['response'] == 511:
            return answer['bin']
        else:
            client_logger.error(f'Не удалось получить ключ собеседника{user}.')

    def send_message(self, to_user, message):
        """Отправка сообщения пользователю"""
        message_dict = {
            'action': 'message',
            'from': self.username,
            'to': to_user,
            'time': time.time(),
            'message_text': message
        }
        client_logger.debug(f'Сформирован словарь сообщения: {message_dict}')

        with socket_lock:
            send_message(self.client_sock, message_dict)
            self.server_response_handler(get_message(self.client_sock))
            client_logger.info(
                f'Отправлено сообщение для пользователя {to_user}')

    def run(self):
        """Основной цикл работы потока"""
        client_logger.debug('Запущен процесс - приёмник собщений с сервера.')
        while self.running:
            time.sleep(1)
            message = None
            with socket_lock:
                try:
                    self.client_sock.settimeout(0.5)
                    message = get_message(self.client_sock)
                except OSError as err:
                    if err.errno:
                        client_logger.critical(
                            f'Потеряно соединение с сервером.')
                        self.running = False
                        self.connection_lost.emit()
                except (ConnectionError, ConnectionAbortedError, ConnectionResetError, json.JSONDecodeError, TypeError):
                    client_logger.debug(f'Потеряно соединение с сервером.')
                    self.running = False
                    self.connection_lost.emit()
                finally:
                    self.client_sock.settimeout(5)

            if message:
                client_logger.debug(f'Принято сообщение с сервера: {message}')
                self.server_response_handler(message)



from setuptools import setup, find_packages

setup(name="client_messager_proj",
      version="0.9.1",
      description="client_messager_proj",
      author="Vovk Vladislav",
      author_email="vovkvladyslav@yandex.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'Crypto']
      )



import logging
import sys

LOGGER = logging.getLogger('server')

class Port:
    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            LOGGER.critical(
            f'Ошибка.Недопустимый порт'
            f'{value}.Порт должен быть в диапазоне от 1024 до 65536')
            sys.exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


import argparse
import os
import sys
sys.path.append('../../../')
import logging
from Crypto.PublicKey import RSA
from server.server.common.decos_logging import log
from client.client.client.client_db import ClientStorage
from PyQt5.QtWidgets import QApplication, QMessageBox
from client.client.client.transport import ClientConnection
from client.client.client.main_window_gui import ClientMainWindow
from client.client.client.start_dialog import UserNameDialog


#-p 8079 -a 95.161.221.138

client_logger = logging.getLogger('client')

@log
def arg_parser():
    """Парсер аргументов командной строки"""
    parser = argparse.ArgumentParser()
    parser.add_argument('addr', default="127.0.0.1", nargs='?')
    parser.add_argument('port', default=7777, type=int, nargs='?')
    parser.add_argument('-n', '--name', default=None, nargs='?')
    parser.add_argument('-p', '--password', default='', nargs='?')
    args = parser.parse_args(sys.argv[1:])
    address = args.addr
    port = args.port
    name = args.name
    password = args.password

    if port < 1023 and port > 65536:
        client_logger.critical(
            f'Ошибка.Недопустимый порт'
            f'{port}.Порт должен быть в диапазоне от 1024 до 65536')
        sys.exit(1)

    return address, port, name, password



if __name__ == '__main__':
    server_address, server_port, client_name, client_password = arg_parser()
    client_logger.debug('Load arguments')

    client_app = QApplication(sys.argv)

    start_dialog = UserNameDialog()

    # запрашиваем имя пользователя
    if not client_name or not client_password:
        client_app.exec_()

        if start_dialog.ok_pressed:
            client_name = start_dialog.client_name.text()
            client_password = start_dialog.client_passwd.text()
            client_logger.debug(f'Using USERNAME = {client_name}, PASSWD = {client_password}.')
        else:
            sys.exit(0)

    client_logger.info(
        f'Запущен клиент: адрес сервера: {server_address} , порт: {server_port}, имя пользователя: {client_name}')

    #  загрузка ключей из файла
    dir_path = os.path.dirname(os.getcwd())
    key_file = os.path.join(dir_path, f'{client_name}.key')

    # если ключ файл с ключем не найден - генерируем новую пару ключей
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())

    client_logger.debug("Keys sucsessfully loaded.")


    try:
        database = ClientStorage(client_name)
        client_logger.debug(f"Создан объект базы данных клиента {client_name}")
    except Exception as err:
        print(f"Ошибка {err}")
        client_logger.critical(f"Ошибка создания объекта базыд данных клиента {err}")

    try:
        transport = ClientConnection(server_port, server_address, database, client_name, client_password, keys)
        client_logger.debug("Transport ready.")
    except Exception:
        message = QMessageBox()
        message.critical(start_dialog, 'Ошибка сервера')
        sys.exit(1)

    transport.setDaemon(True)
    transport.start()

    del start_dialog

    # GUI
    main_window = ClientMainWindow(database, transport, keys)
    main_window.make_connection(transport)
    main_window.setWindowTitle(f'Чат - {client_name}')
    client_app.exec_()

    transport.transport_shutdown()
    transport.join()



